import { AggregateRoot, AggregateID } from '@mobicoop/ddd-library';
import { v4 } from 'uuid';
import { CreateUserProps, UpdateUserProps, UserProps } from './user.types';
import { UserCreatedDomainEvent } from './events/user-created.domain-events';
import { UserUpdatedDomainEvent } from './events/user-updated.domain-events';

export class UserEntity extends AggregateRoot<UserProps> {
  protected readonly _id: AggregateID;

  static create = (create: CreateUserProps): UserEntity => {
    const id = v4();
    const props: UserProps = { ...create };
    const user = new UserEntity({ id, props });

    user.addEvent(
      new UserCreatedDomainEvent({
        aggregateId: id,
        firstName: props.firstName,
        lastName: props.lastName,
        email: props.email,
        phone: props.phone,
      }),
    );
    return user;
  };

  update(props: UpdateUserProps): void {
    this.props.firstName = props.firstName ?? this.props.firstName;
    this.props.lastName = props.lastName ?? this.props.lastName;
    this.props.email = props.email ?? this.props.email;
    this.props.phone = props.phone ?? this.props.phone;
    this.addEvent(
      new UserUpdatedDomainEvent({
        aggregateId: this._id,
        firstName: props.firstName,
        lastName: props.lastName,
        email: props.email,
        phone: props.phone,
      }),
    );
  }

  validate(): void {
    // entity business rules validation to protect it's invariant before saving entity to a database
  }
}
